class MetricCollection(object):

    def __init__(self):
        self.metrics = []

    def append(self, metric):
        self.metrics.append(metric)

    def __str__(self):
        return "\\n".join(str(m) for m in self.metrics)



class Metric(object):

    def __init__(self, measurement):
        self.measurement = measurement
        self.values = {}
        self.tags = dict()
        self.timestamp = None

    def with_timestamp(self, timestamp):
        self.timestamp = timestamp

    def add_tag(self, name, value):
        self.tags[name] = self.__parse_value(value)

    def add_value(self, name, value):
        self.values[name] = self.__parse_value(value)

    def __str__(self):
        protocol = self.measurement
        tags = ["%s=%s" % (key, self.tags[key]) for key in self.tags]
        if len(tags) > 0:
            protocol = "%s,%s" % (protocol, ",".join(tags))

        values = ["%s=%s" % (key, self.values[key]) for key in self.values]
        protocol = "%s %s" % (protocol, ",".join(values))

        if self.timestamp is not None:
            protocol = "%s %d" % (protocol, self.timestamp)

        return protocol

    def __parse_value(self, value):
        if type(value).__name__ == 'int':
            return "%d" % value

        if type(value).__name__ == 'float':
            return "%g" % value

        if type(value).__name__ == 'bool':
            return value and "t" or "f"

        return "\"%s\"" % value



import unittest
from .metric import Metric


class TestMetric(unittest.TestCase):

    def test_metric_without_tags_values_and_timestamp(self):
        metric = Metric("test")

        self.assertEqual("test ", str(metric))

    def test_metric_without_values_and_timestamp(self):
        metric = Metric("test")
        metric.add_tag("tag1", "string")

        self.assertEqual("test,tag1=\"string\" ", str(metric))

    def test_metric_with_tag_value_and_timestamp(self):
        metric = Metric("test")
        metric.add_tag("tag", "string")
        metric.add_value("value", "string")
        metric.with_timestamp(687744000)

        self.assertEqual(
            "test,tag=\"string\" value=\"string\" 687744000", str(metric))

    def test_metric_multiple_tags_without_values_and_timestamp(self):
        metric = Metric("test")
        metric.add_tag("tag1", "string1")
        metric.add_tag("tag2", 1)
        metric.add_tag("tag3", 0.25)
        metric.add_tag("tag4", True)
        metric.add_tag("tag5", False)

        self.assertIn("tag1=\"string1\"", str(metric))
        self.assertIn("tag2=1", str(metric))
        self.assertIn("tag3=0.25", str(metric))
        self.assertIn("tag4=t", str(metric))
        self.assertIn("tag5=f", str(metric))
        self.assertIn("test,tag", str(metric))

    def test_metric_without_tags_and_timestamp(self):
        metric = Metric("test")
        metric.add_value("foo", "bar")

        self.assertEqual("test foo=\"bar\"", str(metric))

    def test_metric_multiple_values_without_tags_and_timestamp(self):
        metric = Metric("test")
        metric.add_value("value1", "string1")
        metric.add_value("value2", 1)
        metric.add_value("value3", 0.25)
        metric.add_value("value4", True)
        metric.add_value("value5", False)

        self.assertIn("value1=\"string1\"", str(metric))
        self.assertIn("value2=1", str(metric))
        self.assertIn("value3=0.25", str(metric))
        self.assertIn("value4=t", str(metric))
        self.assertIn("value5=f", str(metric))
        self.assertIn("test value", str(metric))

    def test_metric_multiple_values_tags_and_timestamp(self):
        metric = Metric("test")
        metric.add_value("value1", "string1")
        metric.add_value("value2", 1)
        metric.add_value("value3", 0.25)
        metric.add_value("value4", True)
        metric.add_value("value5", False)
        metric.add_tag("tag1", "string1")
        metric.add_tag("tag2", 1)
        metric.add_tag("tag3", 0.25)
        metric.add_tag("tag4", True)
        metric.add_tag("tag5", False)
        metric.with_timestamp(687744000)

        self.assertIn("tag1=\"string1\"", str(metric))
        self.assertIn("tag2=1", str(metric))
        self.assertIn("tag3=0.25", str(metric))
        self.assertIn("tag4=t", str(metric))
        self.assertIn("tag5=f", str(metric))
        self.assertIn("value1=\"string1\"", str(metric))
        self.assertIn("value2=1", str(metric))
        self.assertIn("value3=0.25", str(metric))
        self.assertIn("value4=t", str(metric))
        self.assertIn("value5=f", str(metric))
        self.assertIn("test,tag", str(metric))
        self.assertIn(" value", str(metric))
        self.assertIn(" 687744000", str(metric))



import unittest
from .metric import Metric
from .metric_collection import MetricCollection


class TestMetricCollection(unittest.TestCase):

    def test_empty_collection(self):
        collection = MetricCollection()

        self.assertEqual("", str(collection))

    def test_collection_with_one_metric(self):
        metric = Metric("test")
        metric.add_tag("a", 1)
        metric.add_value("b", 2)

        collection = MetricCollection()
        collection.append(metric)

        self.assertEqual("test,a=1 b=2", str(collection))

    def test_collection_with_multiple_metrics(self):
        metric = Metric("test")
        metric.add_tag("a", 1)
        metric.add_value("b", 2)

        metric2 = Metric("test2")
        metric2.add_tag("c", 3)
        metric2.add_value("d", 4)

        collection = MetricCollection()
        collection.append(metric)
        collection.append(metric2)
        collection.append(metric)
        collection.append(metric2)

        self.assertEqual(
            "test,a=1 b=2\\ntest2,c=3 d=4\\ntest,a=1 b=2\\ntest2,c=3 d=4",
            str(collection))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


def read_file(file_name):
    with open(file_name, 'r') as f:
        return f.read()


setuptools.setup(
    name='influx-line-protocol',
    description='Implementation of influxdata line protocol format in python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='0.1.0',
    url='https://github.com/SebastianCzoch/influx-line-protocol',
    author='Sebastian Czoch',
    author_email='sebastian@czoch.pl',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Operating System :: OS Independent',
    ],
    packages=setuptools.find_packages(),
    setup_requires=[
        "flake8"
    ]
)
